@@ -29,6 +29,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "drawpixmaps.h"
 #include "drawtext.h"
 #include "icon.h"
+#include "runlevels.h"
 #include "sound.h"
 #include "typedef.h"
 #include "xdigger.h"
@@ -46,9 +47,11 @@ Bool intakt = False;
 Bool inpause = False, step = False;
 int lastdiggerarrow;
 Bool in_runlevels = False;
+Bool in_LevelEditor = False;
 Bool indiggeridlesequence;
 Bool cheat = False, ones_cheated = False;
 Bool ton_schritt, ton_steine, ton_diamant;
+Bool replay = False;
 
 void ChangePS()
 {
@@ -246,7 +249,7 @@ Bool LoadLevel(int level_number, Bool edit)
           if (edit)
           {
             switch (id)
-            {              
+            {
             case 3:
               flags = F_GHOST_TYPE180;
               break;      
@@ -301,7 +304,7 @@ Bool LoadLevel(int level_number, Bool edit)
   collected_diamonds = 0;
   collected_grounds = 0;
   lefttime = 5000;
-  wait_count = 256/*50*/;
+  if (!replay) wait_count = 256;
 /*   XDefineCursor(display, mainwindow, cursor_digger); */
   return(True);
 } /* loadlevel(int level_number, Bool edit) */
@@ -1012,6 +1015,55 @@ void TimeStep()
   if ((lefttime == 0) && (!digger_death)) DieDigger();
 } /* TimeStep() */
 
+char rec_data[2500];
+int rec_data_pointer;
+
+void RecordStep()
+{
+  char arrow;
+
+  if (lefttime == 0) return;
+
+  if (kb_left) arrow = ADIGGERL;
+  else if (kb_right) arrow = ADIGGERR;
+  else if (kb_up) arrow = ADIGGERU;
+  else if (kb_down) arrow = ADIGGERD;
+  else arrow = ADIGGERN;
+
+  rec_data[rec_data_pointer] = arrow;
+  /*printf("%d ", rec_data_pointer);*/
+  rec_data_pointer++;
+}
+
+void ReplayStep()
+{
+  char s[2];
+
+  if (lefttime == 0) return;
+
+  if (rec_data_pointer % 8 >= 4) strcpy(s, "R"); else strcpy(s, " ");
+  WriteTextStr(s, 39, vertvar, kcf_weiss, kcb_rot);
+
+  kb_left = False; kb_right = False; kb_up = False; kb_down = False;
+
+  switch (rec_data[rec_data_pointer])
+    {
+    case ADIGGERL:
+      kb_left = True;
+      break;
+    case ADIGGERR:
+      kb_right = True;
+      break;
+    case ADIGGERU:
+      kb_up = True;
+      break;
+    case ADIGGERD:
+      kb_down = True;
+      break;
+    }
+  rec_data_pointer++;
+}
+
 void Ton_Bearbeitung_1()
 {
   ton_schritt = False;
@@ -1034,6 +1086,7 @@ void NextStep()
 {
   Ton_Bearbeitung_1();
 
+  if (replay) ReplayStep(); else RecordStep();
   DiggerStep();
   StoneDiamondStep();
   GhostStep();
@@ -1057,6 +1110,8 @@ int takt_zaehler_one_step, takt_zaehler_diamond_blink;
 void ResetZaehler()
 {
   takt_zaehler_one_step = -1; takt_zaehler_diamond_blink = -1;
+  if (!replay) bzero(rec_data, sizeof(rec_data));
+  rec_data_pointer = 0;
 }
 
 Bool DoTakt()
@@ -1161,6 +1216,7 @@ void RunLevels(int von_level, int bis_level)
   lives = 20;
   score = 0;
   new_level_number = von_level;
+  replay = False;
   takt = True;
 
   done = False;
@@ -1169,23 +1225,23 @@ void RunLevels(int von_level, int bis_level)
     {
       if (!XEventsQueued(display, QueuedAfterReading))
  	{
-	  if (takt) 
-	    {
-	      if (DoTakt())
-              {
-                if ((new_level_number) && (bis_level != -1) &&
-                    (new_level_number > bis_level)) done = True;
-                if (alllevelsready) done = True;
-              }
-              else
-              {
-                quit = True;
-                done = True;
-              }
-	    }
-	  XDigger_Idle();
+          if (takt) 
+          {
+            if (DoTakt())
+            {
+              if ((new_level_number) && (bis_level != -1) &&
+                  (new_level_number > bis_level)) done = True;
+              if (alllevelsready) done = True;
+            }
+            else
+            {
+              quit = True;
+              done = True;
+            }
+          }
+          XDigger_Idle();
 	} /* if (!XEventsQueued()) */
-      else 
+      else
 	{
 	  XNextEvent(display, &event);
           if (event.xany.window == iconwindow)
@@ -1251,6 +1307,7 @@ void RunLevels(int von_level, int bis_level)
                   }
 		  break;
 		case XK_Escape:
+		  replay = False;
                   if (lives > 1)
                   {
                     if (!cheat) lives--;
@@ -1281,6 +1338,10 @@ void RunLevels(int von_level, int bis_level)
 		  quit = True;
 		  done = True;
 		  break;
+		case XK_r:
+		  new_level_number = akt_level_number;
+		  replay = True;
+		  break;
 		case XK_x:
                 case XK_m:
                   back = True;
@@ -1347,6 +1408,7 @@ void RunLevels(int von_level, int bis_level)
 	} /* if (!XEventsQueued()) else */
     } /* while (!done) */
   in_runlevels = False;
+  if (takt) setitimer(ITIMER_REAL, &timervalue, 0);
   strcpy(LastArgv, "");
 } /* RunLevels() */
 
@@ -1359,7 +1421,6 @@ void ALookAtTheRoomsTakt()
   setitimer(ITIMER_REAL, &timervalue, 0);
   if (++takt_zaehler_diamond_blink == 3) takt_zaehler_diamond_blink = 0;
 
-
   if ((takt_zaehler_diamond_blink == 1) && (blink))
   {
     DiamondBlinkStep();
@@ -1413,7 +1474,7 @@ void ALookAtTheRooms()
 	}
       if (!XEventsQueued(display, QueuedAfterReading))
 	{
-	  if (takt) 
+	  if (takt)
 	    {
 	      ALookAtTheRoomsTakt();
 	    }
@@ -1504,6 +1565,7 @@ void ALookAtTheRooms()
     } /* while (!done) */
   strcpy(LastArgv, "");
   in_runlevels = False;
+  if (takt) setitimer(ITIMER_REAL, &timervalue, 0);
 } /* ALookAtTheRooms() */
 
 /*---------------------------------------------------------------------------*/
@@ -1530,8 +1592,8 @@ void Edit_SaveLevel(int level_number)
       break;
     case F_GHOST_TYPELR:
       id = 11;
-      break;      
-    }   
+      break;
+    }
     switch (flags & F_GHOST_MOVEMASK)
     {
     case F_GHOST_DOWN:
@@ -1547,13 +1609,16 @@ void Edit_SaveLevel(int level_number)
       d = 3;
       break;
     }
-    if (number_ghosts & 1)
-      file_daten[0x94 + number_ghosts/2] =
-        (file_daten[0x94 + number_ghosts/2] & 0xf0) | (d);
-    else
-      file_daten[0x94 + number_ghosts/2] =
-        (file_daten[0x94 + number_ghosts/2] & 0x0f) | (d << 4);
-    number_ghosts++;
+    if (number_ghosts < MAX_GHOSTS)
+    {
+      if (number_ghosts & 1)
+        file_daten[0x94 + number_ghosts/2] =
+          (file_daten[0x94 + number_ghosts/2] & 0xf0) | (d);
+      else
+        file_daten[0x94 + number_ghosts/2] =
+          (file_daten[0x94 + number_ghosts/2] & 0x0f) | (d << 4);
+      number_ghosts++;
+    }
   }
   
   bzero(file_daten, sizeof(file_daten));
@@ -1636,7 +1701,7 @@ int auswahl_pos[3] = {1, 2, 0},
 void Handle_Tools_ExposeEvent(XExposeEvent *xexpose)
 {
   int i;
-    
+  
   for (i=0; i<24; i++)
   {
     CopyOnePixmap_Window(toolswindow,
@@ -1652,10 +1717,19 @@ void Handle_Tools_ExposeEvent(XExposeEvent *xexpose)
                    (auswahl_pos[i]%4)*16*xpmmag + (i*5+1)*xpmmag,
                    (auswahl_pos[i]/4)*16*xpmmag,
                    4*xpmmag, 4*xpmmag);
-  }  
+  }
   XFlush(display);
 }
 
+int Anzahl_Ghosts()
+{
+  int erg = 0, x, y;
+
+  for (y=0; y<14; y++) for (x=0; x<20; x++)
+    if (field[x][y].id == ID_GHOST) erg++;
+  return erg;
+}
+
 void Handle_ButtonPressEvent_mainwindow(XButtonEvent *xbutton)
 {
   int i=0, x, y;
@@ -1664,7 +1738,7 @@ void Handle_ButtonPressEvent_mainwindow(XButtonEvent *xbutton)
   if (xbutton->button == Button2) i=1;
   if (xbutton->button == Button3) i=2;
   last_pressed_button = i;
-    
+
   x = xbutton->x / (16*xpmmag);
   y = xbutton->y / (16*xpmmag) - (vertvar+1);
 
@@ -1682,14 +1756,21 @@ void Handle_ButtonPressEvent_mainwindow(XButtonEvent *xbutton)
   }
   else
   {
-    FillFieldNothing(&field[x][y]);
-    field[x][y].id = auswahl_id[i];
-    field[x][y].flags = auswahl_flags[i];
-    field[x][y].pixmap_id = id2pixmap_id(field[x][y].id, field[x][y].flags);
-    field[x][y].todraw = True;
-    DrawField();
+    if ((auswahl_id[i] == ID_GHOST) && (Anzahl_Ghosts() >= MAX_GHOSTS))
+    {
+      fprintf(stderr, "%s: You cant have more than 16 ghosts.\n", progname);
+    }
+    else
+    {
+      FillFieldNothing(&field[x][y]);
+      field[x][y].id = auswahl_id[i];
+      field[x][y].flags = auswahl_flags[i];
+      field[x][y].pixmap_id = id2pixmap_id(field[x][y].id, field[x][y].flags);
+      field[x][y].todraw = True;
+      DrawField();
+    }
   }
-  XFlush(display);  
+  XFlush(display);
 }
 
 void Handle_MotionEvent_mainwindow(XMotionEvent *xmotion)
@@ -1706,12 +1787,20 @@ void Handle_MotionEvent_mainwindow(XMotionEvent *xmotion)
   }
   else
   {
-    FillFieldNothing(&field[x][y]);
-    field[x][y].id = auswahl_id[last_pressed_button];
-    field[x][y].flags = auswahl_flags[last_pressed_button];
-    field[x][y].pixmap_id = id2pixmap_id(field[x][y].id, field[x][y].flags);
-    field[x][y].todraw = True;
-    DrawField();
+    if ((auswahl_id[last_pressed_button] == ID_GHOST) &&
+        (Anzahl_Ghosts() >= MAX_GHOSTS))
+    {
+      fprintf(stderr, "%s: You cant have more than 16 ghosts.\n", progname);
+    }
+    else
+    {
+      FillFieldNothing(&field[x][y]);
+      field[x][y].id = auswahl_id[last_pressed_button];
+      field[x][y].flags = auswahl_flags[last_pressed_button];
+      field[x][y].pixmap_id = id2pixmap_id(field[x][y].id, field[x][y].flags);
+      field[x][y].todraw = True;
+      DrawField();
+    }
   }
   XFlush(display);
 }
@@ -1744,6 +1833,7 @@ void LevelEditor()
   Bool done = False;
   
   in_runlevels = True;
+  in_LevelEditor = True;
   strcpy(LastArgv, " (Edit)");
   windowattributes.background_pixel = BlackPixel(display, screen);
   valuemask = CWBackPixel;
@@ -1794,6 +1884,8 @@ void LevelEditor()
     }
     if (!XEventsQueued(display, QueuedAfterReading))
     {
+      XDigger_Idle();
+      XFlush(display);
     }
     else
     {
@@ -1823,9 +1915,11 @@ void LevelEditor()
               {
                 case XK_p:
                   Edit_SaveLevel(akt_level_number);
+                  in_LevelEditor = False;
                   RunLevels(akt_level_number, akt_level_number);
                   new_level_number = akt_level_number;
                   in_runlevels = True;
+                  in_LevelEditor = True;
                   strcpy(LastArgv, " (Edit)");
                   done = quit;
                   break;
@@ -1910,4 +2004,5 @@ void LevelEditor()
   XDestroyWindow(display, toolswindow);  
   strcpy(LastArgv, "");
   in_runlevels = False;
+  in_LevelEditor = False;
 }
