@@ -19,14 +19,17 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include "typedef.h"
 
+#define MAX_GHOSTS 16
+
 extern FieldEntry field[20][14];
 extern Bool intakt;
 extern int score;
 extern Bool ones_cheated;
 extern Bool in_runlevels;
+extern Bool in_LevelEditor;
 extern int fdiggeridlesequence[2][13];
 
-extern void DoTakt();
+extern Bool DoTakt();
 extern void RunLevels(int von_level, int bis_level);
 extern void ALookAtTheRooms();
 extern void LevelEditor();
